
library(dplyr)
library(tidyr)
library(ggplot2)
library(readstata13)
library(reldist)
library(xtable)
library(flux)

d = read.dta13("~/Dropbox/230BSpring2017/PS3/usdina2010.dta")
head(d)
attributes(d)

#############################
# Part (b)
#############################
# For an explanation on how to calculate Gini coef with weights see:
# http://www.victoria.ac.nz/sacl/about/cpf/publications/pdfs/2015-pubs/WP03_2015_Gini_Inequality.pdf


d1 <- arrange(d, peinc ) %>% filter( peinc>=0 ) %>% select(peinc, dweght) %>%
  mutate( 
    dweght = as.numeric(dweght),
    tot.cum = cumsum(dweght * peinc),
    Fcum = cumsum(dweght)
  )

d2 <- arrange(d, poinc ) %>% filter( poinc>=0 ) %>% select(poinc, dweght) %>%
  mutate( 
    dweght = as.numeric(dweght),
    tot.cum = cumsum(dweght * poinc),
    Fcum = cumsum(dweght)
    )


hh = arrange(d, hweal ) %>% filter( hweal>=0 ) %>% select(hweal, dweght) %>%
  mutate( 
    dweght = as.numeric(dweght),
    tot.cum = cumsum(dweght * hweal),
    Fcum = cumsum(dweght)
    ) 
  

# the Gini coefficient:
gini.pre = gini(d1$peinc, weights =  d1$dweght)
gini.post = gini(d2$poinc, weights =  d2$dweght)
gini.hh = gini(hh$hweal, weights =  hh$dweght)

tab = matrix( c(gini.pre,gini.post,gini.hh),ncol=3 )
rownames(tab) = "Gini coefficient"
colnames(tab) = c("Pre-tax national income", "Post-tax national income", "HH net wealth")
xtable(tab)

### Figures
total1 = sum(d1$peinc * d1$dweght)
total.Fcum1 = sum(d1$dweght)

p = ggplot(data.frame(x=d1$Fcum/total.Fcum1,y= d1$tot.cum/total1 ))+
  geom_line(aes(x=x,y=y),size=1)+theme_bw()+
  labs(x="Population)",
       y="Pre-tax national income")

total2 = sum(d2$poinc * d2$dweght)
total.Fcum2 = sum(d2$dweght)

p = ggplot(data.frame(x=d2$Fcum/total.Fcum2,y= d2$tot.cum/total2 ))+
  geom_line(aes(x=x,y=y),size=1)+theme_bw()+
  labs(x="Population)",
       y="Post-tax national income")

total.hh = sum(hh$hweal * hh$dweght)
total.Fcum.hh = sum(hh$dweght)
p = ggplot(data.frame(x=hh$Fcum/total.Fcum.hh,y= hh$tot.cum/total.hh ))+
  geom_line(aes(x=x,y=y),size=1)+theme_bw()+
  labs(x="Population)",
       y="Household net wealth")

# All together
dp = rbind(data.frame(x=hh$Fcum/total.Fcum.hh,y= hh$tot.cum/total.hh, type = "Household wealth" ),
           data.frame(x=d2$Fcum/total.Fcum2,y= d2$tot.cum/total2, type = "Post-tax national income" ),
           data.frame(x=d1$Fcum/total.Fcum1,y= d1$tot.cum/total1, type = "Pre-tax national income" ))

p = ggplot(dp, aes(x=x,y=y, lty = type, col = type  ))+
  geom_line(size=1)+theme_bw()+
  scale_color_grey(start = 0, end = 0.5)+
  theme(legend.position = c(.3,0.85), 
        legend.text = element_text(size=11))+ # legend positvjust=0.5ion
  guides(col = guide_legend(title=""), lty = guide_legend(title=""))+
  labs(x="Population)",
       y="Income")
ggsave(p, file = "~/Dropbox/230BSpring2017/PS3/lorenz_curves.pdf")

#############################
# Part (c)
#############################
# 
# tb = read.csv("~/Dropbox/230BSpring2017/PS3/tb4.csv")
# tb$income = as.numeric(gsub(",","",as.character(tb$income)))
# tb$income.cum = cumsum(tb$income)
# 
# round(2*(0.5-auc(x=tb$quantile/100,y=tb$income.cum/max(tb$income.cum), thresh = NULL, dens = 100)),dig=4)
# 

#############################
# Part (d)
#############################
dd = filter( d, hweal >= 0 & flwag >= 0 )
dd$dweght = dd$dweght/10^5

dd$tax = 0
dd$tax[dd$hweal <= 20*10^6] = dd$hweal[dd$hweal <= 20*10^6] * 0.01 * 0.01 
dd$tax[dd$hweal > 20*10^6] =  (dd$hweal[dd$hweal>20*10^6] - 20 * 10^6) * 0.01 + 20 * 10^6 * 0.01 * 0.01 
tax.wealth = sum(dd$tax * dd$dweght)/10^9

#############################
# Part (e)
#############################
target = sum( dd$ssuicontrib * dd$dweght )
cat("Revenues in billions: ", target/10^9 ,"\n")

f.rev = function(TT){
  dd$pay = 0
  dd$pay[ dd$flwag > TT & dd$flwag <= 106*10^3 ] = 0.13 * dd$flwag[ dd$flwag > TT & dd$flwag <= 106*10^3 ] 
  dd$pay[ dd$flwag > 106*10^3 ] = 0.13 * 106*10^3  
  rev = sum( dd$pay * dd$dweght )
  return(rev)
}

f.min = function(TT){
  r = f.rev(TT) + tax.wealth * 10^9 - target
  return(abs(r))
}

optim.result =optim( par = 10^3, f.min )
print(optim.result)
cat("Desired T: ", optim.result$par, "\n")

#############################
# Part (f) 
#############################

dd$Fcum = cumsum(dd$dweght)/sum(dd$dweght)

dd$tax2 = 0
dd$tax2[ dd$flwag > optim.result$par & dd$flwag <= 106*10^3 ] = 0.13 * dd$flwag[ dd$flwag > optim.result$par & dd$flwag <= 106*10^3 ] 
dd$tax2[ dd$flwag > 106*10^3 ] = 0.13 * 106*10^3 

dd$poinc2 = dd$peinc - dd$tax2 - dd$tax
dd$poinc1 = dd$peinc - dd$ssuicontrib

gini(dd$poinc2, weights =  dd$dweght)
gini(dd$poinc1, weights =  dd$dweght)

# Share botton 50
sum(dd$poinc2[dd$Fcum<=0.5] * dd$dweght[dd$Fcum<=0.5])/10^9 / (sum(dd$poinc1[dd$Fcum<=0.5] * dd$dweght[dd$Fcum<=0.5])/10^9 )


















